;-------------------------------------------------------------------------
; This is the code for a compound usb device consisting of a 4-port hub and
;         compound Human Interface Device compatible with usbhidio stuff from Jan Axelson
; Top-Level adapted from the Cypress Semiconductor Keyboard reference design
;            by Mathew Laibowitz
; Hub code adapted from the Cypress Semiconductor Keyboard reference design
;            by Mathew Laibowitz
; HID code adapted from Jan Axelson's enchid.asm file by Mathew Laibowitz
;
; This is the top-level module of the firmware. This is the only module that
; needs to be assembled. It will call the additional modules of the code.
;
; Design Specifications:
;   [1] USB Rev 1.1
;   [2] USB Chapter 11 HUB Specification (Rev 1.1)
;   [3] CY7C66113 High-Speed USB Controller with Integrated Hub data sheet
;===========================================================================
; File Name: HIDHUB.ASM
;===========================================================================
;
; Development Environment:
; Assembler:    CYASM.EXE (DOS) Version 1.77 (1998)
;               [example:  CYASM HIDHUB.ASM -b -t4]
;               CAUTION: CANNOT have XPAGE instructions fall within a JMP table.
;                        Code does not jump correctly for obvioius reasons.
;                        (See JMP_TBL.ASM and Version note A1.006).
;                        However MUST have XPAGEON!
;
;------------------------------------------------------------------------
; System parameters:
;   Device              CY7C66113 (DAC with 8K EPROM)
;   Clock Speed:                12MHz internal from 6MHz external crystal (PLL)
;                       USB Hub 48MHz
;   Instruction Cycle   Most instructions are 4 to 8 cycles
;                       CLK/4 => 333ns/instruction
;   Instructions/USB Frame Interval (1ms)
;                       12000 Clocks/ms
;                       Avg: 2000 instructions/ms
;
;------------------------------------------------------------------------
; Module Map:HIDHUB.ASM -->    EQUATE.INC
;   (--> = INCLUDE)            MALLOC.INC
;                              MACRO.INC
;                              HUB.INC
;                              HIDMACRO.INC
;
;                   Lower 4K ROM --------------------------------------------
;                               VECTOR.ASM
;                               TIMER.ASM
;                               MAIN.ASM
;                               HIDMAIN.ASM-->  USB.INC
;                                               REGS.INC
;                                               MACROS.INC
;                                               HID_USB.ASM
;                   Upper 4K ROM --------------------------------------------
;                               HUB.ASM -->     JMP_TBL.ASM
;                                               HUB_EP0.ASM --> HUB_EP0C.ASM
;                                                               HUB_EP0S.ASM
;                                               HUB_EP1.ASM
;                                               RESUME.ASM
;                                               SUSPEND.ASM
;                                               PORT_PWR.ASM
;                               HUBDESC.ASM
;
; Module descriptions:
;   main.asm        Overall system control
;   vector.asm      Interrupt vector table (exceptions) and system address ORG
;   timer.asm       Timer based interrupt and control routines
;   hub.asm         Hub support (General hub routines and EQU's)
;   jmp_tbl.asm     Rom JMP and Lookup tables for Hub message parsing
;   hub_ep0.asm     Hub Endpoint 0 support routines
;   hub_ep0c.asm    Hub Endpoint 0 Class request processors
;   hub_ep0s.asm    Hub Endpoint 0 Standard Device request processors
;   hub_ep1.asm     Hub Endpoint 1 Status Change interrupt transfer pipe processor
;   resume.asm      Port and Hub resume control routines
;   suspend.asm     Hub suspend routines
;   port_pwr.asm    Downstream port power control routines
;   equate.inc      Global system EQU's (CPU specific registers and MASKS)
;   malloc.inc      Memory allocation (RAM) EQU's
;   macro.inc       Generic system macro definitions
;   hubdesc.asm     hub descriptor tables for hub
;
;   refer to  hidmain.asm and hid_usb.asm for hid code include files
;
;------------------------------------------------------------------------
    CPU     66113
    INCLUDE "equate.inc"            ; Global system EQU's
    INCLUDE "hub.inc"



;the hub-hid design uses the upper 4 bits of the DAC port for port power,
;and the lower 4 bits of the DAC port for overcurrent detection

 OC_PORT:                   EQU     REG_DAC_DATA
 MASK_OVERCURRENT_INPUTS:   EQU     0fh             ; DAC0-3 overcurrent FLG's
 POWER_PORT:                EQU     REG_DAC_DATA
 MAX_PORTS:                 EQU     05h             ; 5 ports inc. keyboard
 MAX_EMBEDDED_PORTS:        EQU     04h             ; all ports are embedded
 LIMIT_OVERCURRENT:         EQU     20              ; overcurrent debounce limit



    XPAGEON
    INCLUDE ".\hidsrc\hidmacro.inc" ; hid macro definitions
    INCLUDE "malloc.inc"            ; Memory (RAM) allocations
    INCLUDE "macro.inc"             ; Macro library

    ORG LOWER_4K
    INCLUDE "vector.asm"            ; !!! VECTOR "MUST" be first included source (.ASM) module !!!
    INCLUDE "timer.asm"             ; timer module
    INCLUDE "main.asm"              ; main loop
    INCLUDE ".\hidsrc\hidmain.asm"  ; hid code residing in lower 4 k

ROM_KB_END:

;------------------------------------------------------------------------
; Version/System Data
;------------------------------------------------------------------------
    XPAGEOFF
VERSION:    DS "VER 1.0"
DESIGNER:   DS "MAT"
OWNER:      DS "(c)2000 Mat          "
    XPAGEON

;------------------------------------------------------------------------
; Upper 4K of EPROM
; Include most hub support routines to the upper half of code space.
;------------------------------------------------------------------------
    ORG     UPPER_4K
    INCLUDE "hub.asm"                   ; hub module(s) (self includes other modules)
    INCLUDE "hubdesc.asm"
;------------------------------------------------------------------------

    LED_CONTROL:                    ;stub out status led routine, there are no status leds
    RET


;define custom lookup tables for power bits and overcurrent bits

    XPAGEOFF

TABLE_MASK_POWER:                       ; OFF->OR w/ HUB_PORT_POWER, ON->AND ~ w/ HUB_PORT_POWER
    DB      00010000b                   ; Port 1
    DB      00100000b                   ;      2
    DB      01000000b                   ;      3
    DB      10000000b                   ;      4
;(HID)
    DB      00000000b                   ; Port 5 (Virtual Device for HID)
					;   no power required!
TABLE_MASK_OC:                       ; OFF->OR w/ HUB_PORT_POWER, ON->AND ~ w/ HUB_PORT_POWER
    DB      00000001b                   ; Port 1
    DB      00000010b                   ;      2
    DB      00000100b                   ;      3
    DB      00001000b                   ; Port 4
    DB      00000000b                   ; overcurrent mask for hid



; bitmasks for REG_PORT_ENABLE, REG_PORT_SUSPEND, REG_PORT_RESUME
TABLE_PORT_MASK:
    DB      00000001b       ; port 1 (complement to disable or clear)
    DB      00000010b       ;      2
    DB      00000100b       ;      3
    DB      00001000b       ;      4
    DB      00000000b       ;      5
    DB      00000000b       ;      6
    DB      00000000b       ;       7

    XPAGEON;------------------------------------------------------------------------
; In the event firmware gets lost in unused ROM space, give it a place to
; go where it can't do any damage. Fill unused ROM with DI instructions
; and wait for the watch dog to timeout.
; In 8ms (WDT period) and 333ns/DI instruction
;   24000 instrctions in 8ms or 6X the total instruction space!
;   Runaway code needs a place to rest!
;------------------------------------------------------------------------
;DISABLE_INTS:   EQU 70h
;    FILLROM DISABLE_INTS                ; Fill with DI instructions
;
;    ORG     1FDEh                       ; Must be < 1FE0h (??)
;GO_HOME_DOG:
;    JMP     GO_HOME_DOG                 ; Let the dog bring it home!

