;------------------------------------------------------------------------
; PORT_PWR.ASM
;------------------------------------------------------------------------
; Downsteam port power control module
;------------------------------------------------------------------------
VSW_ENBL_BIT:   EQU 010h
;------------------------------------------------------------------------
POWER_OFF_ALL:
; Disable downstream port power for ALL ports.
; Preset all system states to 1 (0 is an invalid state).
; Clears wPortStatus and wPortChange fields for ALL ports
MASK_POWER_OFF:  EQU FFh
;------------------------------------------------------------------------
    write   MASK_POWER_OFF, POWER_PORT; disable PMOSFET's
    MOV     [HUB_PORT_POWER], A         ; HUB_PORT_POWER = A = POWER_OFF
; Set downstream ports to powered-off state (1)
    MOV     A,  MAX_PORTS
    MOV     X, HUB_PORT_STATE_BASE      ; point X to STATE_HUB_PORT0, first port state
PO_LOOP_0:
    PUSH    A
    MOV     A, PORT_POWERED_OFF         ; A = HUB_PORT_POWERED_OFF
    MOV     [X+0], A                    ; set the mem loc
    POP     A
    INC     X                           ; bump pointer
    DEC     A
    JNZ     PO_LOOP_0                   ; do next loc

; clear ALL wPortStatus and wPortChange values
    MOV     A, MAX_PORTS
    ASL
    ASL                                 ; A = 0
    MOV     X, HUB_PORT_STATUS_BASE     ; point X to wPortStatus base address
PO_LOOP_1:
    PUSH    A
    MOV     A,0
    MOV     [X+0], A                    ; clear mem loc
    POP     A
    INC     X                           ; bump pointer
    DEC     A
    JNZ     PO_LOOP_1                   ; do next loc
    RET

;------------------------------------------------------------------------
POWER_OFF_ONE:
; Disable a ports power. Sets selected mask bit in HUB_PORT_POWER to 1
; Set port state to PORT_POWERED_OFF
; On entry: A = port number 0-3
;------------------------------------------------------------------------
    PUSH    A
    INDEX   TABLE_MASK_POWER            ; get port mask
    OR      A, [HUB_PORT_POWER]         ; set mask bit in HUB_PORT_POWER
    MOV     [HUB_PORT_POWER], A         ; save new HUB_PORT_POWER
    write_A POWER_PORT                   ; disable selected PMOSFET
    POP     A
    MOV     X,A
    MOV     A, PORT_POWERED_OFF         ; A = new state
    MOV     [X+HUB_PORT_STATE_BASE], A  ; Set selected downstream port to powered-off state (1)
    SWAP    A,X                         ;get port back in A
    PUSH    A
    ASL                                 ; A*2
    ASL                                 ; A*4
    SWAP    A, X                        ; X = X*4 (offset into wPortStatus base)
    MOV     A,0
    MOV     [X+HUB_PORT_STATUS_BASE],A  ;clear port status bits
    MOV     [X+(HUB_PORT_STATUS_BASE+2)],A  ;clear port change bits
    MOV     A, [X+(HUB_PORT_STATUS_BASE+1)] ; get wPortStatusX_MSB
    AND     A, ~MASK_PORT_POWER         ; clear port power bit
    AND     A,~MASK_PORT_LOW_SPEED           ; clear low speed bit
    MOV     [X+(HUB_PORT_STATUS_BASE+1)], A ; save status
    POP     A
   RET
;------------------------------------------------------------------------
POWER_ON_ONE:
; Enable a ports power. Clears selected mask bit in HUB_PORT_POWER to 0
; Set port state to PORT_DISCONNECTED.
; Also tests to see if this is a request after an OVERCURRENT condition was
; detected. Overcurrent status is reset and change is set if this is an
; attempt to re-power a port.
; On entry: A = port number 0-3
;------------------------------------------------------------------------
    PUSH    A                           ;save port number
   INDEX   TABLE_MASK_POWER            ; get port mask
    CPL     A                           ; complement
    AND     A, [HUB_PORT_POWER]         ; mask off the bit, enabling the MOSFET
    MOV     [HUB_PORT_POWER], A         ; save new HUB_PORT_POWER
    write_A POWER_PORT                  ; enable selected PMOSFET
    POP     A                           ; restore port
    MOV     X,A                         ; X = port
    MOV     A, PORT_DISCONNECTED        ; A = new state
    MOV     [X+HUB_PORT_STATE_BASE], A  ; Set selected downstream port to disconnected state (2)
    SWAP    A,X                         ; A = port
    PUSH    A                           ; save port
    ASL                                 ; A*2
    ASL                                 ; A*4
    SWAP    A, X                        ; X = X*4 (offset into wPortStatus base)
    INC     X                           ; bump to MSB
    MOV     A, [X+HUB_PORT_STATUS_BASE] ; get wPortStatusX_MSB
    OR      A, MASK_PORT_POWER          ; set port power bit
    MOV     [X+HUB_PORT_STATUS_BASE], A ; save status  \
    hid_enable_vsw
    POP     A                           ; restore port
    RET


;------------------------------------------------------------------------
TASK_OVERCURRENT:
; This task services the MIC2527 port power controller overcurrent detect
; flag outputs (FLG, active low).
; Due to "nuisance tripping" on port power assertion (inrush), the FLG outputs
; of the MIC2527 need to be debounced so as not to report overcurrent detects
; to the host on port power assertion. Debounce period is LIMIT_OVERCURRENT in mS.
;------------------------------------------------------------------------
    read    OC_PORT                     ; read overcurrent data
    CPL     A                           ; complement to make active high
    AND     A, MASK_OVERCURRENT_INPUTS  ; mask off all but OC detect bits
    CMP     A,[SHADOW_PORT_OC]          ; compare with last sample
    JZ      TOC_DEBOUNCE                ; if not the same
    MOV     [SHADOW_PORT_OC],A           ; save new sample
    MOV     A,0
    MOV     [HUB_OVERCURRENT_COUNT],A   ; clear debounce counter
    JMP     TASK_OVERCURRENT_END
TOC_DEBOUNCE:
    MOV     A, [HUB_OVERCURRENT_COUNT]  ; get timer incremented counter
    CMP     A, LIMIT_OVERCURRENT        ; expired? (debounce)
    JNZ     TASK_OVERCURRENT_END        ; no, wait for debounce limit
    INC     A
    MOV     [HUB_OVERCURRENT_COUNT],A   ; yes, zero counter

    MOV     A,HUB_PORT_STATUS_BASE      ;
    MOV     X,A                         ; initialize to base of PortStatus
	MOV     A,0                         ; initialize port number
TOC_LOOP:
    PUSH    A                           ; save port
    MOV     A,[X+0]                     ; get port status
    AND     A,MASK_PORT_OVER_CURRENT    ; isolate overcurrent bit
    POP     A                           ; restore port
    PUSH    X                           ; save pointer to wPortStatus
    JNZ     TOC_OVER                    ; currently in overcurrent
TOC_UNDER:
                                        ; if here, were currently not in overcurrent
    MOV     X,A                         ; X now contains port number
    INDEX   TABLE_MASK_OC               ; get overcurrent mask
    AND     A,[SHADOW_PORT_OC]          ;  check overcurrent bit for this port
    MOV     A,X                         ; get port back in A
    JZ      TOC_NEXT                    ; still not in overcurrent, nothing to do!
    CALL    POWER_OFF_ONE               ;shut down port power
    JMP     TOC_CHANGE
TOC_OVER:
                                        ; if here, were currently in overcurrent
    MOV     X,A                         ; X now contains port number
    INDEX   TABLE_MASK_OC               ; get overcurrent mask
    AND     A,[SHADOW_PORT_OC]          ;  check overcurrent bit for this port
    MOV     A,X                         ; get port back in A
    JNZ      TOC_NEXT                   ; still  in overcurrent, nothing to do!
TOC_CHANGE:
    POP     X                           ; restore X
    PUSH    A                           ; save A
    MOV     A,MASK_PORT_OVER_CURRENT     ; get bit mask for change
    XOR      A,[X+0]
    MOV     [X+0],A
    MOV     A,MASK_PORT_OVER_CURRENT     ; get bit mask for change
    OR      A,[X+2]
    MOV     [X+2],A
    POP     A
    JMP     TOC_NEXT1
TOC_NEXT:
    POP     X
TOC_NEXT1:
    INC     A
    INC     X
    INC     X
    INC     X
    INC     X
    CMP     A, MAX_PORTS                ; done?
    JNZ     TOC_LOOP                    ; no, do next port, else fall thru

TASK_OVERCURRENT_END:
    RET

    
