;------------------------------------------------------------------------
; VECTOR.ASM
;------------------------------------------------------------------------
; Interrupt vector table for CY7C66xxx family.
; And system interrupts for hub device.
;
;------------------------------------------------------------------------

; System interrupt vector jump-table
    JMP POWER_ON_RESET              ; Reset (program execution after reset) MAIN.ASM
    JMP USB_BUS_RESET			; USB Bus Reset ISR 	
    JMP INT_128US_TIMER                 ; 128us timer
    JMP INT_1MS_TIMER                   ; 1.024ms timer
    addr_a_isrs                         ; USB Address A endpoint isrs (3)
    JMP INT_HUB_EP0                     ; USB Address B endpoint 0 (Hub, default)
    JMP INT_HUB_EP1                     ; USB Address B endpoint 1 (Hub)
    JMP INT_HUB                         ; hub
    JMP INT_DAC                         ; dac
    JMP INT_GPIO                        ; gpio
    JMP INT_I2C                         ; i2c
; Next address should be 0x1A, start of program space


;(DEBUG)-----------------------------------------------------------------
; The following vectors are used for debugging errant vectors to unused interrupts
; and can be removed and replaced by a single RETI instruction in a
; production design.
;------------------------------------------------------------------------
INT_VOID:                               ; VOID (unused) interrupt vectors
                                        ; Saves a bit mask for errant vectors to unsued ints
                                        ; Comment out vectors as they are written
;------------------------------------------------------------------------


;-------------------------------------------------------------------------
;INT_128US_TIMER:                       ; TIMER.ASM
;INT_1MS_TIMER:                         ; TIMER.ASM
INT_RESERVED:
    MOV     A, [UNUSED_INTS]
    OR      A, 01h
    JMP     INT_VOID_END
INT_A_EP0:
    MOV     A, [UNUSED_INTS]
    OR      A, 02h
    JMP     INT_VOID_END
INT_A_EP1:
    MOV     A, [UNUSED_INTS]
    OR      A, 04h
    JMP     INT_VOID_END
INT_A_EP2:
    MOV     A, [UNUSED_INTS]
    OR      A, 08h
    JMP     INT_VOID_END
;INT_B_EP0:
;INT_B_EP1:
;INT_HUB:
INT_DAC:
;    MOV     A, [UNUSED_INTS]
;    OR      A, 10h
;   JMP     INT_VOID_END
;INT_GPIO:
;    MOV     A, [UNUSED_INTS]
;   OR      A, 20h
;    JMP     INT_VOID_END
INT_I2C:
    MOV     A, [UNUSED_INTS]
    OR      A, 40h
;------------------------------------------------------------------------
INT_VOID_END:
    MOV     [UNUSED_INTS], A
    RETI

;------------------------------------------------------------------------
INT_HUB_EP0:
; Hub Endpoint 0 Interrupt handler (default pipe)
; Hub descriptors and Hub/Port Status and Control are accessible thru
;   this pipe
;------------------------------------------------------------------------
    PUSH    A
    read    REG_EP_B0_MODE                      ; read the mode register
    MOV     [HUB_EP0_TOKEN], A                  ; save the token
    AND     A,SETUP_TOKEN                       ; if a setup packet was received
    JZ     IHT_IN                               ; 
    MOV     A,[HUB_EP0_TOKEN]                   ; 
    AND     A, MASK_ACK                         ; see if it was "acked" 
    JZ      INT_HUB_EP0_EXIT                    ; no, skip processing
    read    REG_EP_B0_COUNT
    AND     A,03fh                               ;check count
    CMP     A,0Ah                               ;must be == 10 (8 bytes + 2 crc)
    JNZ     INT_HUB_EP0_EXIT
    unlock  REG_EP_B0_MODE,NAK_IN_OUT           ; nak in's and out's while dealing with it
    JMP     IHT_PROCESS                         ; go process it
IHT_IN:
    MOV     A,[HUB_EP0_TOKEN]                   ; if an in packet was received
    AND     A,IN_TOKEN
    JZ      IHT_OUT
    MOV     A,[HUB_EP0_TOKEN]
    AND     A, MASK_ACK                         ; see if it was "acked" by the host
    JZ      INT_HUB_EP0_EXIT                    ; no, skip processing
    MOV     A,[HUB_DATA_COUNT]                  ; yes, see if we have more data for host
    CMP     A,0
    JNZ     IHT_PROCESS                         ; yes, go let main loop deal with it
    unlock  REG_EP_B0_MODE,ACK_OUT_STATUS_IN    ; no, we can ignore next IN and ACK the out
    JMP     INT_HUB_EP0_EXIT
IHT_OUT:                                        ; 
    MOV     A,[HUB_EP0_TOKEN]
    AND     A,OUT_TOKEN                         ; if an out packet was received
    JZ      INT_HUB_EP0_EXIT
    unlock  REG_EP_B0_MODE, ACK_OUT_STATUS_IN   ; make sure we are ACKing outs no matter what
    JMP      INT_HUB_EP0_EXIT                   ; 
IHT_PROCESS:
    MOV     A,[HUB_EP0_TOKEN]                   ; if we get here, we will be signalling
    AND     A,MASK_TOKEN                        ; the main loop to handle the ep0 packet
    MOV     [HUB_EP0_TOKEN],A                   ; we've just received
    disable_hub_ep0_int                         ; disable this interrupt
    set_flag    FLAG_HUB_EP0                    ; flag main loop for processing
INT_HUB_EP0_EXIT:                               ; return
    POP     A
    RETI

;------------------------------------------------------------------------
INT_HUB_EP1:
; Hub Endpoint 1 Interrupt handler (Status Change endpoint)
; - port and hub status CHANGE notifications
; - supports interrupt transfers
; Hub and Port Status Change Bitmap (BITMAP_HUB_EP1)
;       a b c d e f g h
;       | | | | | | | |____ hub change detected
;       | | | | | | |______ Port 1 change detected
;       | | | | | |________ Port 2 change detected
;       | | | | |__________ Port 3 change detected
;       | | | |____________ Port 4 change detected
;       | | |______________ Port 5 change detected
;       | |________________ Port 6 change detected
;       |__________________ Port 7 change detected
;------------------------------------------------------------------------
    PUSH    A
    MOV     A, [BITMAP_HUB_EP1]         ; get bitmap
    CMP     A, 0                        ; if 0, no data to send
    JZ      EP1_NO_DATA

    MOV     A,[TEMP]
    PUSH    A
    read    REG_EP_B1_COUNT             ; get current data 0/1 bit setting
    AND     A, MASK_DATATOGGLE          ; keep data 0/1 bit
    XOR     A, MASK_DATATOGGLE          ; toggle the data 0/1 bit
    OR      A, SIZE_HUB_EP1_DATA        ; set byte count
    MOV     [TEMP], A                   ; save to TEMP
    unlock  REG_EP_B1_COUNT, [TEMP]     ; write to EP_B1_COUNT register
    POP     A
    MOV     [TEMP],A    
EP1_NO_DATA:
    POP     A
    RETI

;------------------------------------------------------------------------
INT_HUB:
; Hub Interrupt handler
;------------------------------------------------------------------------
    PUSH    A                           ; save registers used by this int
    PUSH    X


    read    REG_PORT_SPEED              ; read port speed register
    MOV     [SHADOW_PORT_SPEED], A      ; save to shadow register SHADOW_PORT_SPEED
    read    REG_CONNECT_STAT            ; read port connect status register
    MOV     [SHADOW_CONNECT_STAT], A    ; save to shadow register SHADOW_CONNECT_STAT
    CMP     A, [HUB_LAST_CONNECT_STAT]  ; if no change, test for resume events
    JZ      INT_HUB_NO_CONNECT_CHANGE

    set_flag    FLAG_PORT_CHANGE        ; flag main loop for processing
    disable_hub_int                     ; disable, let Task re-enable

    JMP     INT_HUB_END

INT_HUB_NO_CONNECT_CHANGE:
    read    REG_PORT_RESUME             ; get resume register


    MOV     [SHADOW_PORT_RESUME], A     ; save resume register to shadow register
    read    REG_PORT_SUSPEND            ; get suspended ports
    AND     A, [SHADOW_PORT_RESUME]     ; A = suspend AND resume
    JNZ     INT_HUB_RESUME_PENDING      ; if != 0 resume is req'd on a suspended port

; If hub was suspended and resume was detected on an "enabled" port, only
; REG_PORT_RESUME bits will be set, not REG_PORT_SUSPEND. If we got this far,
; was NOT a conn/discon event or a resume while suspended event.
INT_HUB_ENABLED_RESUME:

    MOV     A, 0                        ; else port was not suspended
    MOV     [SHADOW_PORT_RESUME], A     ; clear hub port resume contents
    JMP     INT_HUB_END                 ; and get out (keep int enabled if not

; A suspended DS port has issued resume K signaling and the hub is awake.
; (must be a DEVICE_REMOTE_WAKEUP device)
INT_HUB_RESUME_PENDING:
    MOV     [SHADOW_PORT_RESUME], A     ; save only suspend AND'ed resume bits
    MOV     A, RESUME_START
    MOV     [STATE_RESUME_PENDING], A   ; set start of resume pending



INT_HUB_END:

; test for hub HW disable of DS port (babble detect)
    read    REG_PORT_ENABLE             ; get enable register
    XOR     A, [HUB_LAST_PORT_ENABLE]   ; XOR with last enable
    JZ      IH_ENABLE_OK                ; if 0, no ports disabled
    set_flag    FLAG_PORT_DISABLED      ; else flag main loop to service
IH_ENABLE_OK:
    POP     X                           ; restore registers
    POP     A
    RETI


